@@ -1,20 +1,22 @@
-/*
-  Copyright (C) 2004 Fons Adriaensen <fons.adriaensen@skynet.be>
-    
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+// -------------------------------------------------------------------------
+//
+//  Copyright (C) 2004-2010 Fons Adriaensen <fons@kokkinizita.net>
+//    
+//  This program is free software; you can redistribute it and/or modify
+//  it under the terms of the GNU General Public License as published by
+//  the Free Software Foundation; either version 2 of the License, or
+//  (at your option) any later version.
+//
+//  This program is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//  GNU General Public License for more details.
+//
+//  You should have received a copy of the GNU General Public License
+//  along with this program; if not, write to the Free Software
+//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+//
+// -------------------------------------------------------------------------
 
 
 #include <stdlib.h>
@@ -27,33 +29,34 @@
 #include "messages.h"
 
 
-char *Spectwin::_formats [] = { "%1.0f",  "%2.1f",  "%3.2f",
-                                "%1.0fk", "%2.1fk", "%3.2fk",
-                                "%1.0fM", "%2.1fM", "%3.2fM" };
+const char *Spectwin::_formats [9] = { "%1.0f",  "%2.1f",  "%3.2f",
+                                       "%1.0fk", "%2.1fk", "%3.2fk",
+                                       "%1.0fM", "%2.1fM", "%3.2fM" };
+
+
+const char *Spectwin::_notes[12] = { "C", "C#,Db", "D", "D#,Eb", "E", "F",
+                                     "F#,Gb", "G", "Ab,G#", "A", "Bb,A#", "B" };
 
 
 
-Spectwin::Spectwin (X_window *parent, ITC_ctrl *audio, const char *name) :
+Spectwin::Spectwin (X_window *parent, X_resman *xres, ITC_ctrl *audio) :
     X_window (parent, XPOS, YPOS, XDEF, YDEF, Colors.main_bg), _xs (XDEF), _ys (YDEF), _running (1),
     _audio (audio), _input (-1), _drag (0), _p_ind (-1), _ngx (0), _ngy (0), _fftplan (0), _fftlen (64)
 {
-    int    x, y;
-    X_hints hints;
-    char   s [1024];
+    int      x, y;
+    X_hints  H;
 
     _exitatom = XInternAtom (dpy (), "WM_DELETE_WINDOW", True);
     XSetWMProtocols (dpy (), win (), &_exitatom, 1);
     _exitatom = XInternAtom (dpy (), "WM_PROTOCOLS", True);
 
-    hints.position (XPOS, YPOS);
-    hints.minsize (XMIN, YMIN);
-    hints.maxsize (XMAX, YMAX);
-    hints.rname ("jaaa");
-    hints.rclas ("Jaaa");
-    x_apply (&hints); 
+    H.position (XPOS, YPOS);
+    H.minsize (XMIN, YMIN);
+    H.maxsize (XMAX, YMAX);
+    H.rname (xres->rname ());
+    H.rclas (xres->rclas ());
+    x_apply (&H); 
 
-    sprintf (s, "%s-%s  [%s]", PROGNAME, VERSION, name);
-    x_set_title (s);
     x_add_events (ExposureMask | ButtonMotionMask | ButtonPressMask | ButtonReleaseMask | StructureNotifyMask);
 
     _plotwin = new X_window (this, LMAR, TMAR, XDEF - LMAR - RMAR, YDEF - TMAR - BMAR, Colors.spect_bg);
@@ -71,20 +74,22 @@ Spectwin::Spectwin (X_window *parent, ITC_ctrl *audio, const char *name) :
 
     Bst1.size.x = RMAR - 6;
     y = 60;
-    _butt [BANDW] = new X_tbutton (this, this, &Bst1, x, y, "Bandw",  0, BANDW);
+    _butt [BANDW] = new X_tbutton (this, this, &Bst1, x, y, "Bandw",   0, BANDW);
     y += Bst1.size.y;  
-    _butt [VIDAV] = new X_tbutton (this, this, &Bst1, x, y, "Vid.Av", 0, VIDAV);
+    _butt [VIDAV] = new X_tbutton (this, this, &Bst1, x, y, "Vid.Av",  0, VIDAV);
     y += Bst1.size.y;
-    _butt [HOLD]  = new X_tbutton (this, this, &Bst1, x, y, "Freeze", 0, HOLD);
+    _butt [PEAKH] = new X_tbutton (this, this, &Bst1, x, y, "Pk Hold", 0, PEAKH);
+    y += Bst1.size.y;
+    _butt [FREEZ] = new X_tbutton (this, this, &Bst1, x, y, "Freeze",  0, FREEZ);
 
-    y = 136;
+    y = 153;
     _butt [MCLR]  = new X_tbutton (this, this, &Bst1, x, y, "Clear", 0, MCLR);
     y += Bst1.size.y;
     _butt [MPEAK] = new X_tbutton (this, this, &Bst1, x, y, "Peak",  0, MPEAK);
     y += Bst1.size.y;
     _butt [MNSE]  = new X_tbutton (this, this, &Bst1, x, y, "Noise", 0, MNSE);
 
-    y = 212;
+    y = 229;
     _butt [FMIN]  = new X_tbutton (this, this, &Bst1, x, y, "Min",  0, FMIN);
     y += Bst1.size.y;
     _butt [FMAX]  = new X_tbutton (this, this, &Bst1, x, y, "Max",  0, FMAX);
@@ -93,25 +98,25 @@ Spectwin::Spectwin (X_window *parent, ITC_ctrl *audio, const char *name) :
     y += Bst1.size.y;
     _butt [FSPAN] = new X_tbutton (this, this, &Bst1, x, y, "Span", 0, FSPAN);
 
-    y = 306;
+    y = 323;
     _butt [AMAX]  = new X_tbutton (this, this, &Bst1, x, y, "Max", 0, AMAX);
     y += Bst1.size.y;
     _butt [ASPAN] = new X_tbutton (this, this, &Bst1, x, y, "Range", 0, ASPAN);
 
-    y = 365;
+    y = 383;
     _txt1 = new X_textip (this, this, &Tst1, x, y, RMAR - 6, 18, 16);
     _txt1->x_set_win_gravity (NorthEastGravity);
     _txt1->x_map ();
 
     Bst0.size.x = 15;
     Bst0.size.y = 15;
-    y = 388;
+    y = 405;
     _butt [BTDN] = new X_ibutton (this, this, &Bst0, x + RMAR - 38, y, disp ()->image1515 (X_display::IMG_LT), BTDN);
     _butt [BTUP] = new X_ibutton (this, this, &Bst0, x + RMAR - 23, y, disp ()->image1515 (X_display::IMG_RT), BTUP);
 
     Bst0.size.x = 17;
     Bst0.size.y = 17;
-    y = 435;
+    y = 452;
     _butt [OP1] = new X_tbutton (this, this, &Bst0, x     , y, "1", 0, OP1);
     _butt [OP2] = new X_tbutton (this, this, &Bst0, x + 19, y, "2", 0, OP2);
     _butt [OP3] = new X_tbutton (this, this, &Bst0, x + 38, y, "3", 0, OP3);
@@ -119,12 +124,12 @@ Spectwin::Spectwin (X_window *parent, ITC_ctrl *audio, const char *name) :
 
     Bst0.size.x = 27;
     Bst0.size.y = 17;
-    y = 459;
+    y = 476;
     _butt [NSACT] = new X_tbutton (this, this, &Bst0, XDEF - 30, y, "On", 0, NSACT);
     y += Bst0.size.y + 4;
     _butt [NSLEV] = new X_tbutton (this, this, &Bst1, x, y, "Level", 0, NSLEV);
 
-    y = 503;
+    y = 520;
     _butt [SIACT] = new X_tbutton (this, this, &Bst0, XDEF - 30, y, "On", 0, SIACT);
     y += Bst0.size.y + 4;
     _butt [SILEV] = new X_tbutton (this, this, &Bst1, x, y, "Ampl", 0, SILEV);
@@ -138,6 +143,7 @@ Spectwin::Spectwin (X_window *parent, ITC_ctrl *audio, const char *name) :
     }
 
     _ipbuf = (float *) fftwf_malloc (BUF_LEN * sizeof (float));
+    _fftbuf = (float *) fftwf_malloc (FFT_MAX * sizeof (float));
     _trbuf = (fftwf_complex *) fftwf_malloc ((FFT_MAX / 2 + 9) * sizeof (fftwf_complex));
     _power = new float [FFT_MAX + 1]; //!!!
 
@@ -162,6 +168,7 @@ Spectwin::~Spectwin (void)
     delete _spect;
     delete[] _power;
     fftwf_free (_trbuf);
+    fftwf_free (_fftbuf);
     fftwf_free (_ipbuf);
     XFreePixmap (dpy (), _plotmap);
     XFreeGC (dpy (), _plotgct);
@@ -272,12 +279,29 @@ void Spectwin::handle_callb (int k, X_window *W, _XEvent *E )
 	    }
             else 
      	    {
-		B->set_stat (2);
+		B->set_stat (2); 
+		_butt [PEAKH]->set_stat (0);
                 _spect->_avcnt = 1;
+                _spect->_bits &= ~Spectdata::PEAKH;
 	    }
             break;
 
-        case HOLD:
+        case PEAKH:
+            if (B->stat ())
+     	    {
+		B->set_stat (0);
+                _spect->_bits &= ~Spectdata::PEAKH;
+	    }
+            else 
+     	    {
+		B->set_stat (2);
+		_butt [VIDAV]->set_stat (0);
+                _spect->_bits |=  Spectdata::PEAKH;	
+                _spect->_avcnt = 0;
+            }
+            break;
+
+        case FREEZ:
             if (B->stat ())
      	    {
 		B->set_stat (0);
@@ -341,7 +365,7 @@ void Spectwin::resize (XConfigureEvent *E)
     XSetWindowBackgroundPixmap (dpy (), _plotwin->win (), _plotmap);
     _spect->_npix = _xs - LMAR - RMAR;
     _ngx = _ngy = 0;
-plot_clear ();
+    plot_clear ();
     redraw ();
 }
 
@@ -358,17 +382,17 @@ void Spectwin::redraw (void)
     D.drawstring ("Input", -1);
     D.move (_xs - RMAR + 2, 55);
     D.drawstring ("Analyser", -1);
-    D.move (_xs - RMAR + 2, 131);
+    D.move (_xs - RMAR + 2, 148);
     D.drawstring ("Markers", -1);
-    D.move (_xs - RMAR + 2, 207);
+    D.move (_xs - RMAR + 2, 224);
     D.drawstring ("Frequency", -1);
-    D.move (_xs - RMAR + 2, 301);
+    D.move (_xs - RMAR + 2, 318);
     D.drawstring ("Amplitude", -1);
-    D.move (_xs - RMAR + 2, 430);
+    D.move (_xs - RMAR + 2, 447);
     D.drawstring ("Output", -1);
-    D.move (_xs - RMAR + 2, 472);
+    D.move (_xs - RMAR + 2, 489);
     D.drawstring ("Noise", -1);
-    D.move (_xs - RMAR + 2, 516);
+    D.move (_xs - RMAR + 2, 533);
     D.drawstring ("Sine", -1);
 
     plot_fscale ();
@@ -398,7 +422,9 @@ void Spectwin::bpress (XButtonEvent *E)
 
     if (E->subwindow)
     {
-        if (_butt [MPEAK]->stat ())
+	if      (E->button == Button4) mod_param (true);
+	else if (E->button == Button5) mod_param (false);
+	else if (_butt [MPEAK]->stat ())
 	{
             set_mark (calcfreq (E->x), false);
             if (_spect->_bits & Spectdata::FREEZE) update ();
@@ -488,7 +514,7 @@ void Spectwin::set_fsamp (float fsamp, bool symm)
     _bmax = 2 * fsamp / FFT_MIN;
     set_bw (_bmax / 8);
 
-    _amin = -160;
+    _amin = -200;
     _amax =   10;
     _a0 = -100;
     _a1 =    0;
@@ -505,7 +531,7 @@ void Spectwin::set_input (int i)
 {
     if (_input >= 0) _butt [_input + IP1]->set_stat (0);
     _butt [_input = i]->set_stat (2);
-    if (_spect->_avcnt) _spect->_avcnt = 1;
+    _spect->_bits |= Spectdata::RESET;
     _audio->put_event (EV_MESG, new M_input (_input));
 }
 
@@ -552,6 +578,10 @@ void Spectwin::mod_param (bool inc)
 
     switch (_p_ind)
     {
+    case SILEV:
+	if (inc) set_a_si (_a_si + 0.1f);
+	else     set_a_si (_a_si - 0.1f);
+        break;
     case BANDW: set_bw ((inc ? 2.0 : 0.5) * _bw); break;
     case FMIN:  set_f0 (_f0 + df); break;
     case FMAX:  set_f1 (_f1 + df); break;
@@ -665,8 +695,9 @@ void Spectwin::set_a_ns (float a)
 
 void Spectwin::set_a_si (float a)
 {
-    _a_si = a;
+    _p_val = _a_si = a;
     send_genp ();
+    show_param ();
 }
 
 
@@ -953,9 +984,7 @@ void Spectwin::plot_spect (Spectdata *Z)
 }
 
 
-static char* notes[12] = { "C", "C#,Db", "D", "D#,Eb", "E", "F", "F#,Gb", "G", "Ab,G#", "A", "Bb,A#", "B" };
-
-static void print_note (char *s, float f)
+void Spectwin::print_note (char *s, float f)
 {
     int n;
     
@@ -963,7 +992,7 @@ static void print_note (char *s, float f)
     n = (int)(floorf (f + 0.5f)); 
     f -= n;
     if (n == 12) n = 0;
-    sprintf (s, "  %5s (%+2.0lf)", notes [n], 100.0f * f);  
+    sprintf (s, "  %5s (%+2.0lf)", _notes [n], 100.0f * f);  
 }
 
 
@@ -1050,15 +1079,15 @@ void Spectwin::alloc_fft (Spectdata *S)
     int    k;
     float  b;
 
-    b = 1.41 * _fsamp / S->_bw;;
+    b = 1.41f * _fsamp / S->_bw;;
     k = FFT_MIN;
     while ((k < FFT_MAX) && (k < b)) k *= 2; 
     S->_bw = 2.00 * _fsamp / k;
     if (_fftlen != k)
     {
 	if (_fftplan) fftwf_destroy_plan (_fftplan);
-        if (S->_avcnt) S->_avcnt = 1;
-        _fftplan = fftwf_plan_dft_r2c_1d (_fftlen = k, _ipbuf, _trbuf + 4, FFTW_ESTIMATE);
+        S->_bits |= Spectdata::RESET;
+        _fftplan = fftwf_plan_dft_r2c_1d (_fftlen = k, _fftbuf, _trbuf + 4, FFTW_ESTIMATE);
         _ipmod = k / (2 * INP_LEN);
         if (_ipmod < 1) _ipmod = 1;
     }
@@ -1067,10 +1096,14 @@ void Spectwin::alloc_fft (Spectdata *S)
 
 void Spectwin::handle_mesg (ITC_mesg *M)
 {
-    if (M->type () == M_FSAMP)
+    char s [256];
+
+    if (M->type () == M_JINFO)
     {
-	M_fsamp *Z = (M_fsamp *) M;
+	M_jinfo *Z = (M_jinfo *) M;
         set_fsamp ((float)(Z->_fsamp), false);
+        sprintf (s, "%s-%s  [%s]", PROGNAME, VERSION, Z->_jname);
+        x_set_title (s);
         _ipcnt = 0;
         _audio->put_event (EV_MESG, new M_buffp (_ipbuf, INP_MAX, INP_LEN));     
     }
@@ -1080,7 +1113,7 @@ void Spectwin::handle_mesg (ITC_mesg *M)
 
 void Spectwin::handle_trig ()
 {
-    int i, k;
+    int i, h, k;
     float  a, b, p, s, *pow;
 
     k = ++_ipcnt * INP_LEN;
@@ -1088,14 +1121,15 @@ void Spectwin::handle_trig ()
     if (_ipcnt % _ipmod == 0)
     {
         if (_spect->_bits & Spectdata::FREEZE) return;
-
         if (k < _fftlen) 
 	{
 	    memcpy (_ipbuf + INP_MAX, _ipbuf, k * sizeof (float));
             k += INP_MAX;
 	}
-        fftwf_execute_dft_r2c (_fftplan, _ipbuf + k - _fftlen, _trbuf + 4);
+	memcpy (_fftbuf, _ipbuf + k - _fftlen, _fftlen * sizeof (float));
+        fftwf_execute_dft_r2c (_fftplan, _fftbuf, _trbuf + 4);
         k = _fftlen / 2;
+
         for (i = 1; i <= 4; i++)
 	{
 	    _trbuf [4 - i][0] =  _trbuf [4 + i][0];
@@ -1104,7 +1138,14 @@ void Spectwin::handle_trig ()
 	    _trbuf [4 + k + i][1] = -_trbuf [4 + k - i][1];
 	}
 
+	if (_spect->_bits & Spectdata::RESET)
+	{
+	    _spect->_bits ^= Spectdata::RESET;
+	    memset (_power, 0, (FFT_MAX + 1) * sizeof (float));
+	    if (_spect->_avcnt) _spect->_avcnt = 1;
+	}
         k = _spect->_avcnt; 
+        h = _spect->_bits & Spectdata::PEAKH;
         a = 1.0f;
         if (k)
 	{
@@ -1119,19 +1160,22 @@ void Spectwin::handle_trig ()
 	    p = b * conv0 (_trbuf + 4 + i);
             s += p;
             if (k) *pow += a * (p - *pow);
+	    else if (h) { if (p > *pow) *pow = p; }
             else   *pow = p;
             pow++;
 	    p = b * conv1 (_trbuf + 4 + i);
             s += p;
             if (k) *pow += a * (p - *pow);
+	    else if (h) { if (p > *pow) *pow = p; }
             else   *pow = p;
             pow++;
 	}
         p = b * conv0 (_trbuf + 4 + i);
         s += p;
         if (k) *pow += a * (p - *pow);
+        else if (h) { if (p > *pow) *pow = p; }
         else   *pow = p;
-        _ptot += 0.1 * (0.25 * s - _ptot);
+        _ptot += 0.1f * (0.25f * s - _ptot) + 1e-20f;
 	update ();
     }
 }
@@ -1222,6 +1266,62 @@ void Spectwin::calc_spect (Spectdata *S)
     }    
 }
 
+/*
+void Spectwin::calc_spect_log (Spectdata *S)
+{
+    int   i, j, k;
+    float dc, dd, f0, fd, fc, p, pp, pm;
+    float ff0, ff1;
+
+    ff0 = 10.0f;
+    ff1 = _fsamp / 2;
+
+    dc = _fsamp / (2 * _fftlen);
+    dd = log (ff1 / ff0) / (S->_npix - 1);
+    S->_bits |= Spectdata::YP_VAL | S->_bits | Spectdata::YM_VAL;
+
+    f0 = ff0 * expf (-0.5f * dd);
+    j = (int)(ceil (f0 / dc)); 
+    if (j < 0) j = 0;
+
+    for (i = 0; i < S->_npix; i++)
+    {
+        fd = ff0 * expf (dd * (i + 0.5f));
+        fc = j * dc; 
+        pp = pm = 0;
+        for (k = 0; fc < fd; j++, fc += dc)
+        {
+            if (j > _fftlen) break;
+            p = _power [j];
+            if (p > pp) pp = p; 
+            pm += p;
+            k++;
+        }
+        if (k)
+        {
+            S->_yp [i] = pp;
+            S->_ym [i] = pm / k;
+        }
+        else
+        {
+            S->_yp [i] = -1;
+            S->_ym [i] = -1;
+        }
+    }
+
+    if (S->_bits & Spectdata::MK1_SET)
+    {
+        if (S->_bits & Spectdata::MK1_NSE) calc_noise (&(S->_mk1f), &(S->_mk1p));
+        else calc_peak (&(S->_mk1f), &(S->_mk1p), dd / dc);
+
+    }    
+    if (S->_bits & Spectdata::MK2_SET)
+    {
+        if (S->_bits & Spectdata::MK2_NSE) calc_noise (&(S->_mk2f), &(S->_mk2p));
+        else calc_peak (&(S->_mk2f), &(S->_mk2p), dd / dc);
+    }    
+}
+*/
 
 void Spectwin::calc_noise (float *f, float *p)
 {
